<?php
/**
 * Handle installing and updating 
 * @package Unc
 * @author Nicky Hajal
 */
if (!class_exists("Pdr_Core_Installer")) {
	class Pdr_Core_Installer {
		/**
		 * Check Install
		 */
		public function Pdr_Core_Installer()
		{
			$this->checkInstall();
		}

		/**
		 * Check status of install
		 * @return bool|string false, 'install' or 'update'
		 */
		public function checkInstall()
		{
			$v = get_option('pdrp_version');
			$this->current_version = $v;
			$this->version = $v;
			if (!$v || $v !== PDR_VERSION) {
				$this->install($v);
			}
		}

		/**
		 * Run install!
		 */
		public function install()
		{
			$this->checkTables();
			update_option('pdrp_version', PDR_VERSION);
		}

		/**
		 * Loop through schema, check if tables exist and update if not
		 * @global Wpdb $wpdb 
		 */
		public function checkTables()
		{
			global $wpdb;
			$sdir = PDR_DIR . '/application/schema';
			foreach (scandir($sdir) as $sqlfile) {
				if (!in_array($sqlfile, array('..', '.')) && strpos($sqlfile, '.sql') !== false) {
					$table = lcfirst(str_replace('.sql', '', $sqlfile));
					echo $table . ':';
					if (!$this->tableExists($table)) {
						$wpdb->query($this->sqlFromFile($sdir, $sqlfile));
						echo mysql_error();
					}
					else {
						$this->updateTable($table);
					}
					echo '<br/>';
				}
			}
		}

		/**
		 * Check if a Table Exists
		 * @param string $table Table name without any prefixes
		 * @global Pdr_Core_Utility $PDR_UTIL
		 * @global Wpdb $wpdb
		 * @return boolean
		 */
		public function tableExists($table)
		{
			global $PDR_UTIL, $wpdb;
			$table = $PDR_UTIL->t($table);
			if ($wpdb->get_var("SHOW TABLES LIKE '$table'") == $table) {
				return true;
			}
			else {
				return false;
			}
		}

		/**
		 * Loop through each versions changes to update a table
		 * @param string $table 
		 * @global Pdr_Core_Utility $PDR_UTIL
		 */
		public function updateTable($table)
		{
			global $wpdb, $PDR_UTIL;
			$udir = PDR_DIR . '/application/schema/updates';
			$versions = $PDR_UTIL->scandir($udir, array(&$this, 'compareVersions'));
			foreach ($versions as $version) {
				if ($this->compareVersions($version, $this->current_version) > -1) {
					$sdir = $udir . '/' . $version;
					$sqlfile = ucfirst($table) . '.sql';
					if (file_exists($sdir . '/' . $sqlfile)) {
						$wpdb->query($this->sqlFromFile($sdir, $sqlfile));
					}
				}
			}
		}

		/**
		 * Gets contents of file and replaces necessary variables
		 * @param string $dir Directory of SQL file
		 * @param string $file SQL Filename
		 * @global Pdr_Core_Utility $PDR_UTIL
		 * @return string 
		 */
		private function sqlFromFile($dir, $file)
		{
			global $PDR_UTIL;
			$table = $PDR_UTIL->t(lcfirst(str_replace('.sql', '', $file)));
			$sql = file_get_contents($dir . '/' . $file);
			return str_replace('%table_name%', $table, $sql);
		}

		/**
		 * Compare two versions to determine which is more recent
		 * @param string $latest
		 * @param string $existing
		 * @return boolean 
		 */
		public function compareVersions($latest, $existing = false)
		{
			$existing = $existing ? $existing : PDR_VERSION;
			if ($latest == $existing) {
				return 0;
			}
			$l = explode('.', $latest);		// Latest
			$e = explode('.', $existing);	// Existing
			for ($i = 0; $i < 4; $i++) {
				$l[$i] = isset($l[$i]) ? $l[$i] : 0;
				$e[$i] = isset($e[$i]) ? $e[$i] : 0;
				if ($l[$i] != $e[$i]) {
					return ($l[$i] < $e[$i]) ? -1 : 1;
				}
			}
			return true;
		}	

		/**
		 * Download version from site and install
		 * @global WPFilesystem $wp_filesystem
		 * @global Pdr_Core_Utility $PDR_UTIL
		 * @param string $version
		 * @return type 
		 * @todo Properly impelement this when the time is right
		 */
		public function upgrade($version)
		{
			global $wp_filesystem, $PDR_UTIL;
			
			// Make sure we can do this
			if (!function_exists('curl_init')) {
				return array("success" => 0, "error" => "Curl is not installed or enabled, contact your webhost for help.");
			}

			ob_flush();
			flush();			
			ob_end_flush();

			// Setup
			$PDR_UTIL->cleanTempDir();
			return true;
		}

		/**
		 * Copy an upgraded file to the proper location
		 * @global WPFilesystem $wp_filesystem
		 * @global type $pty
		 * @param type $folder
		 * @param type $from
		 * @param type $to
		 * @return type 
		 * @todo Properly implement this when the time is right
		 */
		private function upgradeFile($folder, $from, $to) 
		{
			global $wp_filesystem, $pty;
			$wp_filesystem->chmod($folder, 0777, true);
			if ($wp_filesystem->exists($to)) {
				$wp_filesystem->delete($to, true);
			}
			// Replace Files
			if (!$wp_filesystem->move($from, $to, true) ) {
				$pty->cleanPtyTempDir();
				$pty->log('Problem moving file', array("from" => $folder, "to" => PTY_DIR, "file" => $file, ""));
				return array("success" => 0, "error" => "There was an error moving " . $file);
			}
			else {
				return true;
			}
		}
	}
}